#include "regex.h"
#include "compile.h"
#include "match.h"
#include <stdbool.h>
#include <stdlib.h>

#define REGEX_SCRATCHPAD_DEFAULT 4096

int Regex_Create(Regex* target, allocator_t* allocator)
{
	if (target == NULL) return EDESTADDRREQ;

	int scratchpad_code = Scratchpad_Create(
		&target->machine_memory,
		REGEX_SCRATCHPAD_DEFAULT,
		allocator
	);
	if (scratchpad_code != EXIT_SUCCESS) {
		return ENOMEM;
	}

	target->options = 0;

	return EXIT_SUCCESS;
}

void Regex_Destroy(Regex* regex)
{
	Scratchpad_Destroy(&regex->machine_memory);

	return;
}

static void _Regex_Reset(Regex* regex)
{
	Scratchpad_Reset(&regex->machine_memory);
	regex->head = NULL;

	return;
}

int Regex_Compile(Regex* regex, StringView source)
{
	if (regex == NULL) return EDESTADDRREQ;
	if (source.source == NULL) return EINVAL;

	_Regex_Reset(regex);
	Internal_Regex_Compile(regex, source);

	return EXIT_SUCCESS;
}

int Regex_FirstMatch(Regex* regex, StringView string, RegexMatch* match)
{
	if (regex == NULL) return EINVAL;
	if (match == NULL) return EDESTADDRREQ;

	int return_code = Regex_MatchHere(regex, string, 0, match);

	return return_code;
}
